#ifndef __SIMIY_LOG_H__

#define __SIMIY_LOG_H__

#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <TCHAR.H>

#include "LinkedList.h"

#define UP_NDEF ('N')	// not checking crc
#define UP_SUCC	('S')	// onSfvSuccess
#define	UP_FAIL ('F')	// onSfvFailed

struct SNode
{
	double	fileID;		// fileID or secs, TCHAR filename[_MAX_PATH]; replacement 
	TCHAR	user[32];
	TCHAR	group[32];
	long	size;
	long	speed;
	int		type;		// type or count
};

// RD2 [fileID]=[user]|[group]|[mod]|[isChecked]|[isFreeFile]|
// 
struct AclNode
{
	double	fileID;
	TCHAR	user[32];
	TCHAR	group[32];
	TCHAR	mod[4];
	TCHAR	checked;
	TCHAR	freefile;
};

struct SfvNode
{
	unsigned long	fileID;
	TCHAR			filename[_MAX_PATH];
	double			crc32;
	bool			checked;
};

class CSimiyLog {
protected:
	bool	m_modified;
	CLinkedList *m_Log;			// KEY=fileID
	CLinkedList *m_RaceUser;	// KEY=user, fileID=file_count, list count = race total user
	CLinkedList *m_RaceGroup;	// KEY=group, fileID=file_count, list count = race total group

	CLinkedList *m_Acl;			// KEY=fileID
	CLinkedList *m_Sfv;

	TCHAR	m_LogPath[_MAX_PATH];
	int		m_SfvUpFileCnt;
	int		m_SfvCount;
	int		m_SfvGood;

	int		m_RaceFilesCount;
	int		m_RaceFilesSize;	// unit = kbytes
	int		m_RaceAvgSpeed;		// unit = kbytes/s
	SNode	*m_RacePeakFile;	// m_Log's one Node: peak speed, peak user

	SNode *GetNodeWithCheckBad(LPCTSTR upFilename);
	void LoadSfvFile( );
	int  SfvCheckCrc32();
	void InitLog();
public:
	CSimiyLog();
	~CSimiyLog();
	bool IsModified() { return m_modified; };
	bool IsEmpty() { return (m_Log->dlcount() == 0); };
	bool ReadLogFile( LPCTSTR file );
	bool WriteLogFile( );

	void Add(LPCTSTR user,
		LPCTSTR group,
		long size,
		long speed,
		LPCTSTR filename,
		char type = UP_NDEF
		);
	void AddWithCheck(LPCTSTR user,
		LPCTSTR group,
		long size,
		long speed,
		LPCTSTR filename,
		char type = UP_NDEF
		);

	void DeleteNode(LPCTSTR file);
	void DeleteNode(LPCTSTR filename,
		TCHAR *user,
		TCHAR *group
		);

	int UserCountAndSpeedAndSize(LPCTSTR user,
		TCHAR *count,
		TCHAR *speed,
		TCHAR *size
		);

	void CalcRaceInfo( );
	bool LoadAclFile(LPCTSTR upDir );
	void MakeMissingFiles( );

	int GetSfvCount();
	int GetSfvGood();

	const SNode *GetRaceUser(int ranking);
	const SNode *GetRaceGroup(int ranking);
	int	GetRaceUsersCnt();
	int GetRaceGroupsCnt();
	int	GetRaceFilesCnt();
	int GetRaceFilesSize();
	int GetRaceAvgSpeed();
	const SNode *GetRacePeakFile();
};

extern CSimiyLog SimiyLog;

#endif