#include "stdafx.h"
#include <stdio.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <direct.h>
#include <sys/stat.h>
#include <winbase.h>
#include <time.h>

#include "SimiyLog.h"
#include "SimiyIni.h"
#include "SimiyUtil.h"
#include "SimiyVPair.h"
#include "SimiySfvChk.h"
#include "SimiyFindFile.h"

#ifndef _tstol
	#define _tstol	_ttol
	#define _tstoi	_ttoi
#endif


LPCTSTR GetProgressBar(int percent, TCHAR BlankChar, TCHAR FillChar)
{
	static TCHAR _bar[128];
	int progress_length = SimiyIni.GetValueInt("GENERAL","PROGRESS_LENGTH", 20);
	int fill_count;

	fill_count = progress_length * percent/100;

	int i = 0;
	for( ; i<fill_count; i++)		_bar[i] = FillChar;
	for( ; i<progress_length; i++)	_bar[i] = BlankChar;

	_bar[i] = _T('\0');

	return _bar;
}

void incomplete_tag(LPCTSTR sfv_count, LPCTSTR good, LPCTSTR percent, bool bComplete)
{
	HANDLE findHandle;
	WIN32_FIND_DATA findData;

	TCHAR search_tag[_MAX_PATH], make_tag[_MAX_PATH];

	LPCTSTR Template =
		SimiyIni.GetValueString("TEMPLATE","INCOMPLETE", "[$percent% DONE]-[$good of $sfv_count files checked]");
	//$good, $sfv_count, $percent, $progressbar
	SimiyVPair.InitVPair();
	SimiyVPair.AddPairs(4, "good", "*", "sfv_count", "*", "percent", "*", "progressbar", "*");
	SimiyVPair.ParsingTemplet(Template, search_tag);

	if( bComplete )
	{
		// make_tag is complete tag
		Template =
			SimiyIni.GetValueString("TEMPLATE","COMPLETE", "[100% COMPLETE]-[#$sfv_count@$total_size files at $avg_speed]");

		//$sfv_count, $total_size, $avg_speed, $race_users, $race_groups, $racewin_user, $racewin_ugrp, $racewin_grp
		TCHAR speed[10], users[10], groups[10];

		_stprintf(speed, "%sps", KByteToStr(SimiyLog.GetRaceAvgSpeed()));
		_stprintf(users, "%d", SimiyLog.GetRaceUsersCnt());
		_stprintf(groups, "%d", SimiyLog.GetRaceGroupsCnt());
		
		const SNode *WinUser = SimiyLog.GetRaceUser(1);

		SimiyVPair.InitVPair();
		SimiyVPair.AddPairs(8, "sfv_count", sfv_count, "total_size", KByteToStr(SimiyLog.GetRaceFilesSize()),
			"avg_speed", speed, "race_users", users, "race_groups", groups,
			"racewin_user", WinUser->user, "racewin_ugrp", WinUser->group,
			"racewin_group", SimiyLog.GetRaceGroup(1)->group
			);
		SimiyVPair.ParsingTemplet(Template, make_tag);
	}
	else
	{
		// make_tag is incomplete tag
		SimiyVPair.InitVPair();
		SimiyVPair.AddPairs(4, "good", good, "sfv_count", sfv_count, "percent", percent, "progressbar",
			GetProgressBar( _ttoi(percent),
				*SimiyIni.GetValueString("GENERAL","PROGRESS_BLNK","-"),
				*SimiyIni.GetValueString("GENERAL","PROGRESS_FILL","#")
				)
			);
		SimiyVPair.ParsingTemplet(Template, make_tag);
	}

	findHandle = FindFirstFile( search_tag, &findData );
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		// deleting incomplete tag & make new tag
		_trename( (LPCTSTR) findData.cFileName, make_tag );

		// deleting dummy incomplete tag
		while( FindNextFile(findHandle, &findData) )
		{
			_trmdir( (LPCTSTR) findData.cFileName );
		}
		FindClose(findHandle);
	}
	else
	{
		// Not found incomplete, make new tag
		_tmkdir( make_tag );
	}
}

void up_tag(LPCTSTR user, LPCTSTR group)
{
	if( user == NULL || *user == _TEXT('\0') ||
		! SimiyIni.GetValueInt("GENERAL","UP_DIR", 1) ) return;

	HANDLE findHandle;
	WIN32_FIND_DATA findData;

	TCHAR uCount[32], uSize[32], uSpeed[32];
	TCHAR search_tag[_MAX_PATH], make_tag[_MAX_PATH];

	// user count
	// Aleady, SimiyLog must be generated.
	int iCount = SimiyLog.UserCountAndSpeedAndSize(user, uCount, uSpeed, uSize);

	LPCTSTR Template =
		SimiyIni.GetValueString("TEMPLATE","UP_DIR", "[UP #$upfiles@$size by $user@$group at $speed]");

	SimiyVPair.AddPairs(5, "user", user, "group", group, "upfiles", "*", "size", "*", "speed", "*");
	SimiyVPair.ParsingTemplet(Template, search_tag);

	SimiyVPair.InitVPair();
	SimiyVPair.AddPairs(5, "user", user, "group", group, "upfiles", uCount, "size", uSize, "speed", uSpeed);
	SimiyVPair.ParsingTemplet(Template, make_tag);

	findHandle = FindFirstFile( search_tag, &findData );
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		if( ! iCount )	// not exist :(
			_trmdir( findData.cFileName );
		else
			_trename( findData.cFileName, make_tag );

		while( FindNextFile(findHandle, &findData) )
		{
			_trmdir( findData.cFileName );	// delete dummy folder
		}
		FindClose(findHandle);
	}
	else if( iCount )
	{
		_tmkdir( make_tag );
	}
}

bool IsSubFolder( LPCTSTR dirname )
{
	int len;

	// cd# cd## - default support
	if( ! _tcsnicmp(dirname, "cd", 2) )
	{
		len = _tcslen(dirname);

		if( (len == 3) && _istdigit( *(dirname + 2) ) )
			return true;

		if( (len == 4) && _istdigit( *(dirname + 2) ) && _istdigit( *(dirname + 3) ) ) 
			return true;
	}

	// check extra sub folders
	TCHAR extra_sub_dirs[128];
	_tcscpy(extra_sub_dirs, SimiyIni.GetValueString("GENERAL","EXTRA_SUB_DIRS"," "));
	TCHAR *token = _tcstok(extra_sub_dirs, " ");
	while( token )
	{
		if( ! _tcsicmp(dirname, token) ) return true;

		token = _tcstok(NULL, " ");
	}
	return false;
}

TCHAR release_name[_MAX_DIR];
TCHAR sub_name[_MAX_DIR];

// create, rename, delete
void incomplete_parent(LPCTSTR sfv_count, LPCTSTR good, LPCTSTR percent, bool bComplete)
{
	if( SimiyIni.GetValueInt("GENERAL","INCOMPLETE_PARENT", 1) == 0 ) return;

	TCHAR parent[_MAX_PATH];
	TCHAR prename[_MAX_PATH];
	TCHAR curname[_MAX_PATH];
	TCHAR dirname[_MAX_DIR + 1];
	TCHAR dirname1[_MAX_DIR + 1];
	TCHAR dirname2[_MAX_DIR + 1];
	TCHAR *pChar;
	
	bool haveSubFolder = false;

	// get folder name
	_tcscpy( prename, SFVCHK.UP_DIR );

	if( (pChar = _tcsrchr( prename, '\\' )) )	// delete '\\'
	{
		*pChar = '\0';
	}
	GetDirName(prename, parent);	// release parent dir
	GetFileName(prename, dirname1);	// release name

	
	if( IsSubFolder(dirname1) )		// sub folder
	{
		_tcscpy( prename, parent );
		if( (pChar = _tcsrchr( prename, '\\' )) )	// delete '\\'
		{
			*pChar = '\0';
		}
		
		_tcscpy(dirname2, dirname1);	// release sub folder name

		GetDirName(prename, parent);	// release parent dir
		GetFileName(prename, dirname1);	// real release name

		haveSubFolder = true;
	}
	else
		*dirname2 = '\0';


	_tcscpy(release_name, dirname1);
	_tcscpy(sub_name, dirname2);


	LPCTSTR Templet;

	if( haveSubFolder )
		Templet = SimiyIni.GetValueString("TEMPLATE","INCOMPLETE_PARENTB", "[--INCOMPLETE--]-[$dirname-$sub]");
	else
		Templet = SimiyIni.GetValueString("TEMPLATE","INCOMPLETE_PARENTA", "[--INCOMPLETE--]-[$dirname]");

	SimiyVPair.InitVPair();
	SimiyVPair.AddPairs(6, "dirname", dirname1, "sub", dirname2, "good", "*", "sfv_count", "*", "percent", "*", "progressbar", "*");
	SimiyVPair.ParsingTemplet(Templet, dirname);
	_stprintf(prename, "%s%s", parent, dirname);	// pre name

	SimiyVPair.InitVPair();
	SimiyVPair.AddPairs(6, "dirname", dirname1, "sub", dirname2, "good", good, "sfv_count", sfv_count, "percent", percent,
		"progressbar", GetProgressBar( _ttoi(percent),
						*SimiyIni.GetValueString("GENERAL","PROGRESS_BLNK","-"),
						*SimiyIni.GetValueString("GENERAL","PROGRESS_FILL","#")
						)
		);
	SimiyVPair.ParsingTemplet( Templet, dirname );
	_stprintf(curname, "%s%s", parent, dirname);	// cur name

	HANDLE findHandle;
	WIN32_FIND_DATA findData;
	findHandle = FindFirstFile( prename, &findData );
	if( findHandle != INVALID_HANDLE_VALUE )
	{
		if( _tcschr( (LPCTSTR)findData.cFileName, _T('\\')) )	// full path
			_tcscpy( prename, (LPCTSTR)findData.cFileName);
		else	// 
			// not full path
			_stprintf( prename, "%s%s", parent, (LPCTSTR)findData.cFileName);

		if(bComplete)
		{
			if( SimiyIni.GetValueInt("GENERAL","INCOMPLETE_PARENT", 1) == 1 )
				_tunlink( prename );
			else
				_trmdir( prename );
		}
		else
			_trename( prename, curname );

		FindClose(findHandle);
	}
	else if( ! bComplete )
	{
		if( SimiyIni.GetValueInt("GENERAL","INCOMPLETE_PARENT", 1) == 1 )
		{
			int fd;
			if( (fd = _topen( curname, _O_WRONLY | _O_CREAT, _S_IREAD | _S_IWRITE )) >= 0)
				close(fd);
 		}
		else
			_tmkdir( curname );

	}
}

void complete_cleaning(void)
{
	TCHAR tmpname[_MAX_PATH];
	CSimiyFindFile findFile;

	// deleting missing files
	if( SimiyIni.GetValueInt("GENERAL","MISSING",1) == 1 ) {
		_stprintf(tmpname, "%s*-missing", SFVCHK.UP_DIR);

		findFile.GetFirst( tmpname );
		findFile.DeleteAll();
	}

	// deleting bad files
	if( SimiyIni.GetValueInt("GENERAL","BAD",1) == 1 ) {
		_stprintf(tmpname, "%s*.bad", SFVCHK.UP_DIR);

		findFile.GetFirst( tmpname );
		findFile.DeleteAll();
	}
}

void sfv_success(LPCTSTR user, LPCTSTR group, LPCTSTR speed, LPCTSTR size, LPCTSTR sfv_count, LPCTSTR good)
{
	int igood, isfv_count, percent;
	TCHAR tmpname[_MAX_PATH];

	// logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	igood = _tstoi( good );
	isfv_count = _tstoi( sfv_count );
	percent = (igood * 100) / isfv_count;

	// add file upload record
	SimiyLog.AddWithCheck(user, group, _ttol(size), _ttol(speed), SFVCHK.UP_FILE, UP_SUCC);

	// update Log File
	if( SimiyLog.IsModified() )
	{
		SimiyLog.WriteLogFile( );
	}

	bool bComplete = (percent == 100);
	if( bComplete ) return;

	TCHAR buf[5];
	_stprintf(buf, "%d", percent);

	// create/rename/delete incomplete tag at parent of release
	incomplete_parent(sfv_count, good, buf, bComplete);

	// 1. create/rename/delete incomplete tag 
	// 2. create complete tag (if complete)
	incomplete_tag(sfv_count, good, buf, bComplete);

	if( SimiyIni.GetValueInt("GENERAL","MISSING",1) ) {
		_stprintf(tmpname, "%s%s-missing", SFVCHK.UP_DIR, SFVCHK.UP_FILE);
		_tunlink( tmpname );
	}
	if( SimiyIni.GetValueInt("GENERAL","BAD",1) == 1 ) {
		_stprintf(tmpname, "%s%s.bad", SFVCHK.UP_DIR, SFVCHK.UP_FILE);
		_tunlink( tmpname );
	}
}

void sfv_failed(LPCTSTR user, LPCTSTR group, LPCTSTR speed, LPCTSTR size)
{
	TCHAR tmpname[_MAX_PATH];

	// logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	// add file upload record
	SimiyLog.AddWithCheck(user, group, _ttol(size), _ttol(speed), SFVCHK.UP_FILE, UP_FAIL);

	switch( SimiyIni.GetValueInt("GENERAL","BAD", 1) )
	{
	case 1:
		if( FileExist( SFVCHK.UP_FILE ) )	// not processed in raidenftpd
		{
			_stprintf(tmpname, "%s.bad", SFVCHK.UP_FILE);
			_tunlink( tmpname );					// delete pre-badfile
			_trename( SFVCHK.UP_FILE, tmpname );	// rename file to .bad
		}
		break;
	case 2:
		// delete up file & bad file
		_tunlink(SFVCHK.UP_FILE);
		_stprintf(tmpname, "%s.bad", SFVCHK.UP_FILE);
		_tunlink(tmpname);

		SimiyLog.DeleteNode( SFVCHK.UP_FILE );
		break;
	}

	// update Log File
	if( SimiyLog.IsModified() )
	{
		SimiyLog.WriteLogFile( );
	}
}

void sfv_uploaded(LPCTSTR sfv_count, LPCTSTR good)
{
	int igood, isfv_count, percent;

	igood = _tstoi( good );
	isfv_count = _tstoi( sfv_count );
	if( isfv_count == 0 )
		percent = 0;
	else
		percent = (igood * 100) / isfv_count;

	// not add file upload record -> in file_uploaded

	bool bComplete = (percent == 100);
	if( bComplete ) return;

	/*
	GetExtName( SFVCHK.UP_FILE, ext );
	if( _tcsicmp( ext, ".sfv" ) == 0 )
	{
		TCHAR sfv_count[16], sfv_good[16];
		_stprintf( sfv_count, "%d", SimiyLog.GetSfvCount() );
		_stprintf( sfv_good, "%d", SimiyLog.GetSfvGood() );

		sfv_uploaded( sfv_count, sfv_good );
	}
	*/
	//logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	TCHAR buf[5];
	_stprintf(buf, "%d", percent);

	// create/delete incomplete tag at parent of release
	incomplete_parent(sfv_count, good, buf, bComplete);

	// 1. create/delete incomplete tag 
	// 2. create complete tag (if complete)
	incomplete_tag(sfv_count, good, buf, bComplete);
	
	if( SimiyIni.GetValueInt("GENERAL","MISSING", 1) == 1 )
	{
		// create missing files
		SimiyLog.MakeMissingFiles();

		// update Log File
		if( SimiyLog.IsModified() )
		{
			SimiyLog.WriteLogFile( );
		}
	}
}

void file_uploaded(LPCTSTR user, LPCTSTR group, LPCTSTR speed, LPCTSTR size)
{
	// logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	if( FileExist( SFVCHK.UP_FILE ) )
	{
		// add file upload record
		SimiyLog.AddWithCheck(user, group, _ttol(size), _ttol(speed), SFVCHK.UP_FILE, UP_NDEF);
	}

	// update Log File
	if( SimiyLog.IsModified() )
	{
		SimiyLog.WriteLogFile( );
	}

	up_tag(user, group);
}

void file_deleted()
{
	// logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	TCHAR user[32] = "", group[32] = "";

	// delete file upload record
	// for modifing up_tag, seek user_name
	SimiyLog.DeleteNode(SFVCHK.UP_FILE, user, group);

	// update Log File
	if( SimiyLog.IsModified() )
	{
		SimiyLog.WriteLogFile( );	// if node count == 0, delete
	}

	// modify up_tag
	up_tag(user, group);

	TCHAR tmpname[_MAX_EXT+1];
	// sfv file is deleted
	// missing, bad, incomplete tag, complete tag
	// all sfv-progress file must be deleted
	GetExtName(SFVCHK.UP_FILE, tmpname);

	if( ! _tcsicmp(tmpname, ".sfv") )
	{
		CSimiyFindFile findFile;
		TCHAR search_tag[_MAX_PATH];

		// delete Incomplete Tag
		LPCTSTR Template =
			SimiyIni.GetValueString("TEMPLATE","INCOMPLETE", "[$percent% DONE]-[$good of $sfv_count files checked]");
		SimiyVPair.InitVPair();
		SimiyVPair.AddPairs(4, "good", "*", "sfv_count", "*", "percent", "*", "progressbar", "*");
		SimiyVPair.ParsingTemplet(Template, search_tag);

		findFile.GetFirst( search_tag );
		findFile.DeleteAll();

		// delete incomplete parent
		SimiyVPair.AddPairs(2, "dirname", "*", "sub", "*");	// add two variable
		
		TCHAR parent[_MAX_PATH];
		TCHAR dirname[_MAX_PATH];

		GetDirDivide(SFVCHK.UP_DIR, parent, search_tag);

		if( IsSubFolder( search_tag ) )
		{
			GetDirDivide(parent, parent, dirname);
		}

		Template = SimiyIni.GetValueString("TEMPLATE","INCOMPLETE_PARENTA", "[--INCOMPLETE--]-[$dirname]");
		SimiyVPair.ParsingTemplet(Template, dirname);

		_stprintf(search_tag, "%s%s", parent, dirname);
		
		findFile.GetFirst( search_tag );
		findFile.DeleteAll();

		Template = SimiyIni.GetValueString("TEMPLATE","INCOMPLETE_PARENTB", "[--INCOMPLETE--]-[$dirname-$sub]");
		SimiyVPair.ParsingTemplet(Template, dirname);
		
		_stprintf(search_tag, "%s%s", parent, dirname);

		findFile.GetFirst( search_tag );
		findFile.DeleteAll();

		// delete complete tag
		Template =
			SimiyIni.GetValueString("TEMPLATE","COMPLETE", "[100% COMPLETE]-[#$sfv_count@$total_size files at $avg_speed]");

		SimiyVPair.InitVPair();
		SimiyVPair.AddPairs(8, "sfv_count", "*", "total_size", "*", "avg_speed", "*",
			"race_users", "*", "race_groups", "*", "racewin_user", "*", "racewin_ugrp", "*", "racewin_group", "*");
		SimiyVPair.ParsingTemplet(Template, search_tag);

		findFile.GetFirst( search_tag );
		findFile.DeleteAll();

		// delete message
		Template = SimiyIni.GetValueString("GENERAL","MESSAGE","");
		if( Template && *Template != _T('\0') )
		{
			_stprintf(search_tag, "%s%s", SFVCHK.UP_DIR, Template);
			_tunlink( search_tag );
		}

		// delete missing & bad file
		complete_cleaning();
	}
}

void MakeMessageFile( )
{
	LPCTSTR message = SimiyIni.GetValueString("GENERAL", "MESSAGE", "");

	if( message == NULL || *message == _T('\0') ) return;

	FILE *fp = _tfopen( message,  "wb");

	if( fp == NULL)
	{
		SimiyIni.SfvChkLogAppend( "MakeMessageFile() fopen failed - %s", strerror(errno) );
		return;
	}
	// hidden file
	//SetFileAttributes( message, FILE_ATTRIBUTE_HIDDEN );

	const SNode *node;

	TCHAR tmpname[_MAX_PATH];

	if( *sub_name == '\0' )
		_stprintf(tmpname, "%s", release_name);
	else
		_stprintf(tmpname, "%s (%s)", release_name, sub_name);

	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	_ftprintf(fp, "|                           Release Info                              |\r\n");
	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	_ftprintf(fp, "| Rel Name   : %-54s |\r\n", tmpname);

	_stprintf(tmpname, "%d files, %d KB, %d KBps, %d user(s), %d group(s)", 
		SimiyLog.GetRaceFilesCnt(), SimiyLog.GetRaceFilesSize(), SimiyLog.GetRaceAvgSpeed(),
		SimiyLog.GetRaceUsersCnt(), SimiyLog.GetRaceGroupsCnt()
		);

	_ftprintf(fp, "| Total      : %-54s |\r\n", tmpname);

	node = SimiyLog.GetRaceUser(1);
	_stprintf(tmpname, "%s@%s (%d files, %d KB, %d KB/s)", node->user, node->group, node->type, node->size, node->speed);
	
	_ftprintf(fp, "| Race win   : %-54s |\r\n", tmpname);

	node = SimiyLog.GetRacePeakFile();
	_stprintf(tmpname, "%d KBytes/s (by %s@%s)", node->speed, node->user, node->group);
	
	_ftprintf(fp, "| Peak speed : %-54s |\r\n", tmpname);

	long t;
	time(&t);
	struct tm *pTm = localtime( &t );
	_stprintf(tmpname, "%04d/%02d/%02d %02d:%02d:%02d",	pTm->tm_year + 1900, pTm->tm_mon + 1, pTm->tm_mday,
		pTm->tm_hour, pTm->tm_min, pTm->tm_sec);

	_ftprintf(fp, "| Race end   : %-54s |\r\n", tmpname);

	int i, j, cnt;
	int totalSize = SimiyLog.GetRaceFilesSize();

	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	_ftprintf(fp, "|                           Racer  Info                               |\r\n");
	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	_ftprintf(fp, "|           Racer         | Files |      KBytes    |   %%    |  KB/sec |\r\n");
	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	for(i=1; i <= SimiyLog.GetRaceUsersCnt(); i++)
	{
		node = SimiyLog.GetRaceUser(i);
		_stprintf(tmpname, "%s@%s", node->user, node->group); _ftprintf(fp,"|  %02d. %-17s  ", i, tmpname);
		_ftprintf(fp, "| %4df ", node->type);
		_ftprintf(fp, "|    %8d    ", node->size);
		_ftprintf(fp, "| %5.1f%% ", (double)node->size * 100 / totalSize);
		_ftprintf(fp, "|  %5d  |\r\n", node->speed);
		_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	}
	_ftprintf(fp, "|                           Group  Info                               |\r\n");
	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	_ftprintf(fp, "|      Group     | Racers | Files |      KBytes    |   %%    |  KB/sec |\r\n");
	_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	for(i=1; i <= SimiyLog.GetRaceGroupsCnt(); i++)
	{
		node = SimiyLog.GetRaceGroup(i);
		cnt = 0;
		for(j=1; j <= SimiyLog.GetRaceUsersCnt(); j++)
		{
			if( ! _tcscmp( SimiyLog.GetRaceUser(j)->group, node->group ) ) cnt ++;
		}
		_ftprintf(fp,"|  %02d. %-8s  ", i, node->group);
		_ftprintf(fp, "|  %4d  ", cnt);
		_ftprintf(fp, "| %4df ", node->type);
		_ftprintf(fp, "|    %8d    ", node->size);
		_ftprintf(fp, "| %5.1f%% ", (double)node->size * 100 / totalSize);
		_ftprintf(fp, "|  %5d  |\r\n", node->speed);
		_ftprintf(fp, "+---------------------------------------------------------------------+\r\n");
	}

	fclose(fp);
}

void sfv_complete( )
{
	// logfile open error
	if( !SimiyLog.ReadLogFile( SFVCHK.LOG_PATH ) ) return;

	SimiyLog.CalcRaceInfo( );

	TCHAR buf[10];
	_stprintf(buf, "%d", SimiyLog.GetRaceFilesCnt());

	incomplete_parent(buf, buf, "100", true);
	incomplete_tag(buf, buf, "100", true);

	complete_cleaning();

	MakeMessageFile( );
}